﻿using Hotline.Caching.Interfaces;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Snapshot;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Enums.User;
using Hotline.Snapshot.Interfaces;
using Hotline.ThirdAccountDomainServices;
using Hotline.ThirdAccountDomainServices.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Dependency;

namespace Hotline.Application.Snapshot;
public class SnapshotThirdAccountSupplier : IThirdAccountDomainService, IScopeDependency
{
    private readonly IGuiderInfoRepository _guiderInfoRepository;
    private readonly IThirdAccountRepository _thirdAccountRepository;
    private readonly ICitizenRepository _citizenRepository;
    private readonly IVolunteerRepository _volunteerRepository;
    private readonly ISnapshotUserInfoRepository _snapshotUserInfoRepository;
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;

    public SnapshotThirdAccountSupplier(IGuiderInfoRepository guiderInfoRepository, IThirdAccountRepository thirdAccountRepository, ICitizenRepository citizenRepository, IVolunteerRepository volunteerRepository, ISnapshotUserInfoRepository snapshotUserInfoRepository, ISystemSettingCacheManager systemSettingCacheManager)
    {
        _guiderInfoRepository = guiderInfoRepository;
        _thirdAccountRepository = thirdAccountRepository;
        _citizenRepository = citizenRepository;
        _volunteerRepository = volunteerRepository;
        _snapshotUserInfoRepository = snapshotUserInfoRepository;
        _systemSettingCacheManager = systemSettingCacheManager;
    }

    public async Task<List<Claim>> GetClaimAsync(ThirdAccount account, List<Claim> claims, CancellationToken token)
    {
        return await Task.FromResult(claims);
    }

    public async Task<Dictionary<string, object>> GetLoginOutDataAsync(ThirdAccount thirdAccount, Dictionary<string, object> dicOutData, CancellationToken cancel)
    {
        var isVolunteer = await _volunteerRepository.IsVolunteerAsync(thirdAccount.PhoneNumber);
        var user = await _snapshotUserInfoRepository.Queryable().Where(m => m.ThirdAccountId == thirdAccount.Id).FirstAsync();
        dicOutData.Add("IsVolunteer", isVolunteer);
        dicOutData.Add("InvitationCode", user.InvitationCode);
        return dicOutData;
    }

    public async Task<ThirdTokenDto> GetThirdParameterAsync(ThirdTokenDto thirdDto, CancellationToken token)
    {
        if (thirdDto.ThirdType == EThirdType.WeChat)
        {
            thirdDto.AppId = _systemSettingCacheManager.WxOpenAppId;
            thirdDto.Secret = _systemSettingCacheManager.WxOpenAppSecret;
        }
        return await Task.FromResult(thirdDto);
    }

    public async Task<string> RegisterAsync(ThirdAccount thirdAccount, CancellationToken token)
    {
        thirdAccount.CitizenType = EReadPackUserType.Citizen;
        var guider = await _guiderInfoRepository.GetByPhoneNumberAsync(thirdAccount.PhoneNumber);
        if (guider != null)
        {
            thirdAccount.CitizenType = EReadPackUserType.Guider;
            thirdAccount.UserId = guider.Id;
        }
        else
        {
            var citizen = await _citizenRepository.Queryable().Where(m => m.PhoneNumber == thirdAccount.PhoneNumber).FirstAsync();
            thirdAccount.UserId = citizen?.Id;
        }
        return await _thirdAccountRepository.AddAsync(thirdAccount);
    }
}

